# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.encounter_id import EncounterId
from ....commons.types.page_token import PageToken
from ....commons.types.task_id import TaskId
from ....commons.types.user_id import UserId
from ..commons.types.task_status import TaskStatus
from ..commons.types.task_type import TaskType
from .raw_client import AsyncRawV3Client, RawV3Client
from .types.task import Task
from .types.task_actions import TaskActions
from .types.task_create_v_3 import TaskCreateV3
from .types.task_page import TaskPage
from .types.task_sort_options import TaskSortOptions
from .types.task_update_v_3 import TaskUpdateV3

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV3Client
        """
        return self._raw_client

    def get_actions(self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None) -> TaskActions:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TaskActions

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.tasks.v_3.get_actions(
            task_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get_actions(task_id, request_options=request_options)
        return _response.data

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TaskPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        status : typing.Optional[TaskStatus]

        task_type : typing.Optional[TaskType]

        categories : typing.Optional[str]
            Only return tasks with categories that match one in this comma-separated list.

        updated_since : typing.Optional[dt.datetime]
            Only return tasks updated on or after this date-time

        encounter_id : typing.Optional[EncounterId]
            Only return tasks associated with this encounter

        search_term : typing.Optional[str]
            Query tasks by encounter_id, claim_id, task_id, or external_id

        assigned_to_id : typing.Optional[UserId]
            Only return tasks assigned to this user

        date_of_service_min : typing.Optional[dt.date]
            The minimum date of service for the linked encounter

        date_of_service_max : typing.Optional[dt.date]
            The maximum date of service for the linked encounter

        billing_provider_npi : typing.Optional[str]
            The NPI of the billing provider associated with the task's claim

        sort : typing.Optional[TaskSortOptions]
            Defaults to updated_at:desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TaskPage

        Examples
        --------
        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.tasks.v_3.get_multi()
        """
        _response = self._raw_client.get_multi(
            limit=limit,
            page_token=page_token,
            status=status,
            task_type=task_type,
            categories=categories,
            updated_since=updated_since,
            encounter_id=encounter_id,
            search_term=search_term,
            assigned_to_id=assigned_to_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            billing_provider_npi=billing_provider_npi,
            sort=sort,
            request_options=request_options,
        )
        return _response.data

    def get(self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None) -> Task:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.tasks.v_3.get(
            task_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(task_id, request_options=request_options)
        return _response.data

    def create(self, *, request: TaskCreateV3, request_options: typing.Optional[RequestOptions] = None) -> Task:
        """
        Parameters
        ----------
        request : TaskCreateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.tasks.resources.commons import TaskType
        from candid.resources.tasks.resources.v_3 import TaskCreateV3

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.tasks.v_3.create(
            request=TaskCreateV3(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                task_type=TaskType.CUSTOMER_DATA_REQUEST,
                description="description",
                work_queue_id="work_queue_id",
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def update(
        self, task_id: TaskId, *, request: TaskUpdateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> Task:
        """
        Parameters
        ----------
        task_id : TaskId

        request : TaskUpdateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.tasks.resources.v_3 import TaskUpdateV3

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.tasks.v_3.update(
            task_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=TaskUpdateV3(),
        )
        """
        _response = self._raw_client.update(task_id, request=request, request_options=request_options)
        return _response.data


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV3Client
        """
        return self._raw_client

    async def get_actions(
        self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TaskActions:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TaskActions

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.tasks.v_3.get_actions(
                task_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_actions(task_id, request_options=request_options)
        return _response.data

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TaskPage:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        status : typing.Optional[TaskStatus]

        task_type : typing.Optional[TaskType]

        categories : typing.Optional[str]
            Only return tasks with categories that match one in this comma-separated list.

        updated_since : typing.Optional[dt.datetime]
            Only return tasks updated on or after this date-time

        encounter_id : typing.Optional[EncounterId]
            Only return tasks associated with this encounter

        search_term : typing.Optional[str]
            Query tasks by encounter_id, claim_id, task_id, or external_id

        assigned_to_id : typing.Optional[UserId]
            Only return tasks assigned to this user

        date_of_service_min : typing.Optional[dt.date]
            The minimum date of service for the linked encounter

        date_of_service_max : typing.Optional[dt.date]
            The maximum date of service for the linked encounter

        billing_provider_npi : typing.Optional[str]
            The NPI of the billing provider associated with the task's claim

        sort : typing.Optional[TaskSortOptions]
            Defaults to updated_at:desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TaskPage

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.tasks.v_3.get_multi()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            limit=limit,
            page_token=page_token,
            status=status,
            task_type=task_type,
            categories=categories,
            updated_since=updated_since,
            encounter_id=encounter_id,
            search_term=search_term,
            assigned_to_id=assigned_to_id,
            date_of_service_min=date_of_service_min,
            date_of_service_max=date_of_service_max,
            billing_provider_npi=billing_provider_npi,
            sort=sort,
            request_options=request_options,
        )
        return _response.data

    async def get(self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None) -> Task:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.tasks.v_3.get(
                task_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(task_id, request_options=request_options)
        return _response.data

    async def create(self, *, request: TaskCreateV3, request_options: typing.Optional[RequestOptions] = None) -> Task:
        """
        Parameters
        ----------
        request : TaskCreateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.tasks.resources.commons import TaskType
        from candid.resources.tasks.resources.v_3 import TaskCreateV3

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.tasks.v_3.create(
                request=TaskCreateV3(
                    encounter_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    task_type=TaskType.CUSTOMER_DATA_REQUEST,
                    description="description",
                    work_queue_id="work_queue_id",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def update(
        self, task_id: TaskId, *, request: TaskUpdateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> Task:
        """
        Parameters
        ----------
        task_id : TaskId

        request : TaskUpdateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Task

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.tasks.resources.v_3 import TaskUpdateV3

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.tasks.v_3.update(
                task_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=TaskUpdateV3(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(task_id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ServiceIdQualifier(str, enum.Enum):
    EAN_UCC_13 = "EN"
    """
    EAN/UCC - 13
    """

    EAN_UCC_8 = "EO"
    """
    EAN/UCC - 8
    """

    HIBC = "HI"
    """
    HIBC (Health Care Industry Bar Code) Supplier Labeling Standard Primary Data Message
    """

    NDC_5_4_2_FORMAT = "N4"
    """
    National Drug Code in 5-4-2 Format
    """

    CUSTOMER_ORDER_NUMBER = "ON"
    """
    Customer Order Number
    """

    GTIN = "UK"
    """
    GTIN 14-digit Data Structure
    """

    UCC_12 = "UP"
    """
    UCC - 12
    """

    _UNKNOWN = "__SERVICEIDQUALIFIER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ServiceIdQualifier":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ean_ucc_13: typing.Callable[[], T_Result],
        ean_ucc_8: typing.Callable[[], T_Result],
        hibc: typing.Callable[[], T_Result],
        ndc_5_4_2_format: typing.Callable[[], T_Result],
        customer_order_number: typing.Callable[[], T_Result],
        gtin: typing.Callable[[], T_Result],
        ucc_12: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ServiceIdQualifier.EAN_UCC_13:
            return ean_ucc_13()
        if self is ServiceIdQualifier.EAN_UCC_8:
            return ean_ucc_8()
        if self is ServiceIdQualifier.HIBC:
            return hibc()
        if self is ServiceIdQualifier.NDC_5_4_2_FORMAT:
            return ndc_5_4_2_format()
        if self is ServiceIdQualifier.CUSTOMER_ORDER_NUMBER:
            return customer_order_number()
        if self is ServiceIdQualifier.GTIN:
            return gtin()
        if self is ServiceIdQualifier.UCC_12:
            return ucc_12()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....coverages.resources.v_1.types.member_info import MemberInfo
from .encounter import Encounter
from .provider import Provider


class EligibilityRequest(UniversalBaseModel):
    """
    An object representing the data for an eligibility request.
    """

    payer_id: str = pydantic.Field()
    """
    Supported payer ID values can be found [here](https://www.stedi.com/healthcare/network).
    """

    provider: Provider
    subscriber: MemberInfo = pydantic.Field()
    """
    The primary policyholder for the insurance plan or a dependent with a unique member ID. <Note>If a dependent has a unique member ID, include their information here and leave dependent undefined.</Note>
    """

    dependent: typing.Optional[MemberInfo] = pydantic.Field(default=None)
    """
    <Note>If a dependent has a unique member ID, include their information as subscriber and leave this field undefined.</Note>
    """

    encounter: typing.Optional[Encounter] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....service_lines.resources.v_2.types.denial_reason_content import DenialReasonContent
from .....x_12.resources.v_1.types.claim_adjustment_reason_code import ClaimAdjustmentReasonCode
from .....x_12.resources.v_1.types.remittance_advice_remark_code import RemittanceAdviceRemarkCode


class ServiceLineAdjudicationCreate(UniversalBaseModel):
    denial_reason: typing.Optional[DenialReasonContent] = pydantic.Field(default=None)
    """
    Will be treated as a denial if present
    """

    insurance_allowed_amount_cents: typing.Optional[int] = None
    insurance_paid_amount_cents: typing.Optional[int] = None
    deductible_amount_cents: typing.Optional[int] = None
    coinsurance_amount_cents: typing.Optional[int] = None
    copay_amount_cents: typing.Optional[int] = None
    carcs: typing.List[ClaimAdjustmentReasonCode]
    rarcs: typing.List[RemittanceAdviceRemarkCode]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .payer_id import PayerId
from .payer_name import PayerName
from .payer_uuid import PayerUuid


class PayerIdentifier_PayerInfo(UniversalBaseModel):
    type: typing.Literal["payer_info"] = "payer_info"
    payer_id: PayerId
    payer_name: PayerName

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PayerIdentifier_PayerUuid(UniversalBaseModel):
    value: PayerUuid
    type: typing.Literal["payer_uuid"] = "payer_uuid"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


PayerIdentifier = typing.Union[PayerIdentifier_PayerInfo, PayerIdentifier_PayerUuid]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.user_id import UserId
from .eligibility_check_status import EligibilityCheckStatus
from .service_type_code import ServiceTypeCode


class EligibilityCheckMetadata(UniversalBaseModel):
    check_id: str
    service_code: ServiceTypeCode
    status: EligibilityCheckStatus
    initiated_by: UserId
    initiated_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....diagnoses.types.diagnosis_id import DiagnosisId
from .drug_identification import DrugIdentification
from .service_line_denial_reason import ServiceLineDenialReason
from .test_result import TestResult


class ServiceLineUpdate(UniversalBaseModel):
    edit_reason: typing.Optional[str] = None
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line, factoring in quantity. If `procedure_code` is updated and this is not, the system will attempt
    to set it based on chargemasters entries and the service line's quantity. For example, if a single unit has an entry of 100 cents and 2
    units were rendered, the `charge_amount_cents` will be set to 200, if this field is unfilled.
    """

    diagnosis_id_zero: typing.Optional[DiagnosisId] = None
    diagnosis_id_one: typing.Optional[DiagnosisId] = None
    diagnosis_id_two: typing.Optional[DiagnosisId] = None
    diagnosis_id_three: typing.Optional[DiagnosisId] = None
    drug_identification: typing.Optional[DrugIdentification] = None
    denial_reason: typing.Optional[ServiceLineDenialReason] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    837p Loop2300, SV105. If your organization does not intend to submit claims with a different place of service at the service line level, this field should not be populated. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    units: typing.Optional[ServiceLineUnits] = None
    procedure_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    If `procedure_code` is updated, and `charge_amount_cents` is not, then `charge_amount_cents` will be set by the system.
    """

    quantity: typing.Optional[Decimal] = pydantic.Field(default=None)
    """
    String representation of a Decimal that can be parsed by most libraries.
    A ServiceLine quantity cannot contain more than one digit of precision.
    Example: 1.1 is valid, 1.11 is not.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.
    """

    date_of_service: typing.Optional[dt.date] = pydantic.Field(default=None)
    """
    date_of_service must be defined on either the encounter or the service lines but not both.
    """

    end_date_of_service: typing.Optional[dt.date] = None
    test_results: typing.Optional[typing.List[TestResult]] = pydantic.Field(default=None)
    """
    Contains a list of test results. Test result types may map to MEA-02 on the 837-P (ex: Hemoglobin, Hematocrit).
    No more than 5 MEA-02 test results may be submitted per service line.
    Updating test results utilizes PUT semantics, so the test results on the service line will be set to whatever inputs are provided.
    """

    has_epsdt_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-11 on the 837-P and Box 24H on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    has_family_planning_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.decimal import Decimal
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.procedure_modifier import ProcedureModifier
from .....commons.types.service_line_units import ServiceLineUnits
from .....encounter_providers.resources.v_2.types.ordering_provider import OrderingProvider
from .drug_identification import DrugIdentification
from .test_result import TestResult


class ServiceLineCreate(UniversalBaseModel):
    modifiers: typing.Optional[typing.List[ProcedureModifier]] = None
    has_epsdt_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-11 on the 837-P and Box 24H on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    has_family_planning_indicator: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Maps to SV1-12 on the 837-P and Box 24I on the CMS-1500.
    If the value is true, the box will be populated with "Y". Otherwise, the box will not be populated.
    """

    procedure_code: str
    quantity: Decimal = pydantic.Field()
    """
    String representation of a Decimal that can be parsed by most libraries.
    A ServiceLine quantity cannot contain more than one digit of precision.
    Example: 1.1 is valid, 1.11 is not.
    """

    units: ServiceLineUnits
    charge_amount_cents: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total amount charged for this service line taking quantity into account. For example, if a single unit
    costs 100 cents and 2 units were rendered, the `charge_amount_cents` should be 200. Should be greater than or
    equal to 0.
    """

    diagnosis_pointers: typing.List[int] = pydantic.Field()
    """
    Indices (zero-indexed) of all the diagnoses this service line references
    """

    drug_identification: typing.Optional[DrugIdentification] = None
    place_of_service_code: typing.Optional[FacilityTypeCode] = pydantic.Field(default=None)
    """
    837p Loop2300, SV105. If your organization does not intend to submit claims with a different place of service at the service line level, this field should not be populated. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    A free-form description to clarify the related data elements and their content. Maps to SV1-01, C003-07 on the 837-P.
    """

    date_of_service: typing.Optional[dt.date] = None
    end_date_of_service: typing.Optional[dt.date] = None
    ordering_provider: typing.Optional[OrderingProvider] = pydantic.Field(default=None)
    """
    Required when the service or supply was ordered by a provider who is different than the rendering provider for this service line.
    If not required by this implementation guide, do not send.
    """

    test_results: typing.Optional[typing.List[TestResult]] = pydantic.Field(default=None)
    """
    Contains a list of test results. Test result types may map to MEA-02 on the 837-P (ex: Hemoglobin, Hematocrit).
    No more than 5 MEA-02 test results may be submitted per service line.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .non_insurance_payer_payment import NonInsurancePayerPayment
from .non_insurance_payer_payment_create import NonInsurancePayerPaymentCreate
from .non_insurance_payer_payment_id import NonInsurancePayerPaymentId
from .non_insurance_payer_payment_sort_field import NonInsurancePayerPaymentSortField
from .non_insurance_payer_payments_page import NonInsurancePayerPaymentsPage

__all__ = [
    "NonInsurancePayerPayment",
    "NonInsurancePayerPaymentCreate",
    "NonInsurancePayerPaymentId",
    "NonInsurancePayerPaymentSortField",
    "NonInsurancePayerPaymentsPage",
]
